-- minimum amount of time between two LootLink scans of the auction house (1 hour = 3600 seconds)
LOOTLINK_TIME_BETWEEN_AUCTION_SCANS = 3600;

-- prevents the user from seeing what the LootLink does at the Auction House
LootLink_SearchAuctionHouse = 1;

LootLink_Auction_Browsing = false;

-- whether we're done getting links with the current page (perhaps not necessary, but I don't know if OnUpdate is ran in its own thread...)
LootLink_Auction_DoneWithCurrentPage = true;

LootLink_Auction_CurrentPage = 0;
LootLink_Auction_TimeSinceLastScan = 0;

LootLink_Saved_AuctionFrame_OnShow = nil;
LootLink_Saved_AuctionFrameBrowse_OnEvent = nil;
LootLink_Saved_CanSendAuctionQuery = nil;


LootLink_AuctionArray = {};

LOOTLINK_MAXIMUM_NUMBER_OF_REMEMBERED_THINGS = 10;

LootLink_LastSellCursorInfo = nil;
LootLink_Saved_ShowContainerSellCursor = nil;

LootLink_RefreshOnShow = 0;

LootLinkExtension_Saved_LootLink_OnShow = nil;
LootLinkExtension_Saved_LootLink_ShowTooltip = nil;
LootLinkExtension_Saved_LootLinkItemButton_OnEnter = nil;
LootLinkExtension_Saved_LootLinkItemButton_OnLeave = nil;

-- Use quantity retrieval code
LootLink_UseQuantityToDeterminePrice = 0;



function LootLinkExtension_OnLoad()
	this:RegisterEvent("MERCHANT_SHOW");
	this:RegisterEvent("MERCHANT_UPDATE");
	this:RegisterEvent("TOOLTIP_ADD_MONEY");

	LootLinkExtension_CosmosRegister();

	if ( Cosmos_RegisterChatCommand ) then
		local LootLinkMainCommands = {"/lootlink","/ll"};
		Cosmos_RegisterChatCommand (
			"LOOTLINK_MAIN_COMMANDS", -- Some Unique Group ID
			LootLinkMainCommands, -- The Commands
			LootLink_SlashCommandHandler,
			COSMOS_LOOTLINK_CHAT_COMMAND_INFO -- Description String
		);
	else
		-- Register our slash command
		SLASH_LOOTLINK1 = "/lootlink";
		SLASH_LOOTLINK2 = "/ll";
		SlashCmdList["LOOTLINK"] = function(msg)
			LootLink_SlashCommandHandler(msg);
		end
	end
	
	LootLink_AuctionFrame_HookFunctions();
	LootLink_HookTooltipMoneyFunctions();
	LootLinkExtension_HookLootLinkFunctions();
end


function LootLinkExtension_CosmosRegister()
	local name = "LootLink";
	local shortdesc = "Show items";
	local desc = "Allows you to browse all items the UI has seen since you started using LootLink.";
	if ( Cosmos_RegisterButton ) then 
		Cosmos_RegisterButton(
			TEXT(COSMOS_LOOTLINK_NAME),
			TEXT(COSMOS_LOOTLINK_SHORT_DESC), 
			TEXT(COSMOS_LOOTLINK_LONG_DESC), 
			TEXT(COSMOS_LOOTLINK_BUTTON_TEXTURE), 
			ToggleLootLink
		);
	end
	if ( Cosmos_RegisterConfiguration ) then 
		Cosmos_RegisterConfiguration(
			"COS_LOOTLINK",
			"SECTION",
			TEXT(COSMOS_LOOTLINK_NAME),
			TEXT(COSMOS_LOOTLINK_LONG_DESC)
		);
		Cosmos_RegisterConfiguration(
			"COS_LOOTLINK_HEADER",
			"SEPARATOR",
			TEXT(COSMOS_LOOTLINK_NAME),
			TEXT(COSMOS_LOOTLINK_LONG_DESC)
		);
		Cosmos_RegisterConfiguration(
			"COS_LOOTLINK_SEARCHAUCTIONHOUSE",
			"CHECKBOX",
			TEXT(COSMOS_LOOTLINK_SEARCH_AUCTION_HOUSE),
			TEXT(COSMOS_LOOTLINK_SEARCH_AUCTION_HOUSE_INFO),
			LootLink_SetSearchAuctionHouse,
			LootLink_SearchAuctionHouse
		);
		Cosmos_RegisterConfiguration(
			"COS_LOOTLINK_REFRESHONSHOW",
			"CHECKBOX",
			TEXT(COSMOS_LOOTLINK_REFRESH_ON_OPEN),
			TEXT(COSMOS_LOOTLINK_REFRESH_ON_OPEN_INFO),
			LootLink_SetRefreshOnShow,
			LootLink_RefreshOnShow
		);
		--[[
		Cosmos_RegisterConfiguration(
			"COS_LOOTLINK_TOGGLEBUTTON",
			"BUTTON",
			"Show LootLink",
			"Shows the LootLink window",
			ToggleLootLink,
			0,
			0,
			0,
			0,
			"Show"
		);
		]]--
	end
end

function LootLinkExtension_OnEvent(event)
	if( ( event == "MERCHANT_SHOW" ) or ( event == "MERCHANT_UPDATE" ) ) then
		LootLink_ScanMerchant();
	elseif ( event == "TOOLTIP_ADD_MONEY" ) then
		LootLink_HandleTooltipAddMoney(arg1, arg2);
	end
end

function LootLink_ScanMerchant()
	local size;
	local slotid;
	local link;
	
	size = GetMerchantNumItems();
	for slotid = size, 1, -1 do
		local name, _, merchantBuyPrice, quantity = GetMerchantItemInfo(slotid);
		link = GetMerchantItemLink(slotid);
		if( link ) then
			LootLink_ProcessLinks(link);
		end
		if ((ItemLinks[name] ~= nil) and (merchantBuyPrice ~= nil)) then
			local realBuyPrice = merchantBuyPrice / quantity;
			if(ItemLinks[name].buyPrice ~= nil ) then
				
				if(ItemLinks[name].buyPrice > realBuyPrice) then
  					ItemLinks[name].buyPrice = realBuyPrice;
				end
			else
				ItemLinks[name].buyPrice = realBuyPrice;
			end
		end
	end
end



function LootLink_SetSearchAuctionHouse(toggle)
	if ( toggle ~= LootLink_SearchAuctionHouse ) then
		LootLink_SearchAuctionHouse = toggle;
	end
end

function LootLink_Auction_StartScan()
	LootLink_Auction_TimeSinceLastScan = GetTime();
	LootLink_Auction_DoneWithCurrentPage = true;
	LootLink_GetAllAuctionItems();
end

function LootLink_AuctionFrame_HookFunctions()
	if ( not LootLink_Saved_AuctionFrame_OnShow ) then
		LootLink_Saved_AuctionFrame_OnShow = AuctionFrame_OnShow;
		AuctionFrame_OnShow = LootLink_AuctionFrame_OnShow;
	end
	if ( not LootLink_Saved_AuctionFrameBrowse_OnEvent ) then
		LootLink_Saved_AuctionFrameBrowse_OnEvent = AuctionFrameBrowse_OnEvent;
		AuctionFrameBrowse_OnEvent = LootLink_AuctionFrameBrowse_OnEvent;
	end
	if ( not LootLink_Saved_CanSendAuctionQuery ) then
		LootLink_Saved_CanSendAuctionQuery = CanSendAuctionQuery;
		CanSendAuctionQuery = LootLink_CanSendAuctionQuery;
	end
end

-- this is called in the BrowseSearchButton:OnUpdate handler which is why it works
function LootLink_CanSendAuctionQuery()
	local value = LootLink_Saved_CanSendAuctionQuery();
	if ( LootLink_Auction_Browsing ) then
		if ( (value) and ( LootLink_Auction_DoneWithCurrentPage ) ) then
			LootLink_GetNextAuctionPage();
		end
		return false;
	else
		return value;
	end
end

-- hides the search results while scanning is going on
function LootLink_AuctionFrameBrowse_OnEvent()
	LootLink_GetAllCurrentBrowsedItems();
	if ( not LootLink_Auction_Browsing ) then
		LootLink_Saved_AuctionFrameBrowse_OnEvent();
	end
end

function LootLink_AddAuctionItemInfo(id)
	if ( gcinfo() > 28000) then return; end
	local name, texture, count, quality, canUse, level, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, owner =  GetAuctionItemInfo("list", id);
	if ( not name ) then return; end
	
	local currentBid = bidAmount;
	if ( bidAmount == 0 ) then
		currentBid = minBid;
	end
	local currentBuyout = buyoutPrice;
	local element = LootLink_AuctionArray[name];
	if ( not element ) then
		element = {};
		element.bidsIndex = 1;
		element.bids = {};
		element.buyoutsIndex = 1;
		element.buyouts = {};
	end
	if ( currentBid > 0 ) then
		local size = getn(element.bids);
		if ( size < LOOTLINK_MAXIMUM_NUMBER_OF_REMEMBERED_THINGS ) then
			table.insert(element.bids, currentBid);
		else
			if ( element.bidsIndex > LOOTLINK_MAXIMUM_NUMBER_OF_REMEMBERED_THINGS ) then
				element.bidsIndex = 1;
			end
			element.bids[element.bidsIndex] = currentBuyout;
			element.bidsIndex = element.bidsIndex + 1;
		end
	end
	if ( currentBuyout > 0 ) then
		local size = getn(element.buyouts);
		if ( size < LOOTLINK_MAXIMUM_NUMBER_OF_REMEMBERED_THINGS ) then
			table.insert(element.buyouts, currentBuyout);
		else
			if ( element.buyoutsIndex > LOOTLINK_MAXIMUM_NUMBER_OF_REMEMBERED_THINGS ) then
				element.buyoutsIndex = 1;
			end
			element.buyouts[element.buyoutsIndex] = currentBuyout;
			element.buyoutsIndex = element.buyoutsIndex + 1;
		end
	end
	LootLink_AuctionArray[name] = element;
end

function LootLink_GetAllCurrentBrowsedItems()
	local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");
	local link = nil;
	for i = 1, numBatchAuctions do
		link = GetAuctionItemLink("list", i);
		if ( link ) then
			LootLink_ProcessLinks(link);
			--LootLink_AddAuctionItemInfo(i);
		end
		if ( gcinfo() > 28000) then break; end
	end
	LootLink_Auction_DoneWithCurrentPage = true;
end

function LootLink_GetNextAuctionPage()
	if ( LootLink_Auction_Browsing ) then
		if ( not AuctionFrame:IsVisible() ) then
			LootLink_Auction_Browsing = false;
			return;
		end
		local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");
		
		local numberOfPages = floor(totalAuctions / NUM_AUCTION_ITEMS_PER_PAGE);
		
		if ( LootLink_Auction_CurrentPage < numberOfPages ) then
			LootLink_Auction_CurrentPage = LootLink_Auction_CurrentPage + 1;
			LootLink_Auction_DoneWithCurrentPage = true;
			QueryAuctionItems("", "", "", nil, nil, nil, LootLink_Auction_CurrentPage, false, nil);
			if ( LootLink_Auction_CurrentPage == 0 ) then
				BrowseNoResultsText:SetText("LootLink is searching for new items in the auction house, please be patient.");
			else
				BrowseNoResultsText:SetText(format("LootLink is currently searching page %d/%d, please be patient.", LootLink_Auction_CurrentPage+1, numberOfPages+1));
			end
			LootLink_Auction_DoneWithCurrentPage = false;
		else
			BrowseNoResultsText:SetText("LootLink has finished searching - thank you for your patience.");
			LootLink_Auction_Browsing = false;
		end
	else
		LootLink_Auction_DoneWithCurrentPage = true;
	end
end

function LootLink_GetAverageAuctionInfoValue(elementValueArray)
	local value = 0;
	for k, v in elementValueArray do
		value = value + v;
	end
	value = value / getn(elementValueArray);
	return value;
end

function LootLink_GetCoinValue(value)
	value = floor(value);
	local gold = floor(value / 10000);
	value = value - gold * 10000;
	local silver = floor(value / 100);
	value = value - silver * 100;
	local str = "";
	if ( gold > 0 ) then
		if ( strlen(str) > 0 ) then str = str.." "; end
		str = str..gold.." g";
	end
	if ( silver > 0 ) then
		if ( strlen(str) > 0 ) then str = str.." "; end
		str = str..silver.." s";
	end
	if ( value > 0 ) then
		if ( strlen(str) > 0 ) then str = str.." "; end
		str = str..value.." c";
	end
	return str;
end

function LootLink_GetAuctionInfo(element)
	if ( element ) then
		local bidsValue = LootLink_GetAverageAuctionInfoValue(element.bids);
		local buyoutsValue = LootLink_GetAverageAuctionInfoValue(element.buyouts);
		local str = "";
		if ( bidsValue > 0 ) then
			str = str.."BidAvg: "..LootLink_GetCoinValue(bidsValue);
			if ( buyoutsValue > 0 ) then
				str = str.." ";
			end
		end
		if ( buyoutsValue > 0 ) then
			str = str.."BuyoutAvg: "..LootLink_GetCoinValue(buyoutsValue);
		end
		return str;
	else
		return "";
	end
end


function LootLink_AddAuctionInfoToTooltip(element, tooltipName)
	if ( not tooltipName ) then
		tooltipName = "LootLinkTooltip";
	end
	local textName = nil;
	local obj = nil;
	local value = nil;
	local patched = false;
	local tooltip = getglobal(tooltipName);
	for i = 2, 20 do
		textName = tooltipName.."TextLeft"..i;
		obj = getglobal(textName);
		if ( obj ) then
			value = obj:GetText();
			if ( ( not value ) or ( strlen(value) <= 0) or ( not obj:IsVisible() ) ) then
				if ( element ) then
					obj:SetText(LootLink_GetAuctionInfo(element));
					obj:Show();
					if ( tooltip ) then
						local height = tooltip:GetHeight();
						height = height * ( i / (i-1) );
						tooltip:SetHeight(height);
					end
					patched = true;
					break;
				end
			end
		end
	end
	if ( not patched ) then
		for i = 2, 20 do
			textName = tooltipName.."TextLeft"..i;
			obj = getglobal(textName);
			if ( obj ) then
				value = obj:GetText();
				if ( ( value ) and ( strlen(value) > 0 ) and 
				( ( strfind(value, "BidAvg") ) or ( strfind(value, "BuyoutAvg") ) ) ) then
					obj:Hide();
				end
			end
		end
	end
end

function LootLink_SlashCommandHandler(msg)
	ToggleLootLink();
end

function LootLink_GetAllAuctionItems()
	LootLink_Auction_Browsing = true;
	LootLink_Auction_CurrentPage = -1;
end

function LootLink_AuctionFrame_OnShow()
	local curTime = GetTime();
	if ( ( LootLink_Auction_TimeSinceLastScan == 0 ) or 
		( ( LootLink_Auction_TimeSinceLastScan + LOOTLINK_TIME_BETWEEN_AUCTION_SCANS ) < curTime ) ) then
		if ( LootLink_SearchAuctionHouse == 1 ) then
			LootLink_Auction_StartScan();
		end
	end
	LootLink_Saved_AuctionFrame_OnShow();
end

function LootLink_HookTooltipMoneyFunctions()
	if ( not LootLink_Saved_ShowContainerSellCursor ) then
		LootLink_Saved_ShowContainerSellCursor = ShowContainerSellCursor;
		ShowContainerSellCursor = LootLink_ShowContainerSellCursor;
	end
end

function LootLink_ShowContainerSellCursor(bagId, itemId)
	LootLink_LastSellCursorInfo = {};
	LootLink_LastSellCursorInfo.bag = bagId;
	LootLink_LastSellCursorInfo.item = itemId;
	if ( LootLink_Saved_ShowContainerSellCursor ) then
		return LootLink_Saved_ShowContainerSellCursor(bagId, itemId);
	end
end

function LootLink_HandleTooltipAddMoneyGetQuantity()
	if ( not LootLink_LastSellCursorInfo ) then
		return -1;
	else
		local texture, itemCount = GetContainerItemInfo(LootLink_LastSellCursorInfo.bag, LootLink_LastSellCursorInfo.item);
		LootLink_LastSellCursorInfo = nil;
		if ( texture ) then
			return itemCount;
		else
			return -1;
		end
	end
end

function LootLink_HandleTooltipAddMoney(tooltipName, money)
	if ( not InRepairMode() ) then
		local text = getglobal(tooltipName.."TextLeft1");
		if ( text ) then
			local name = text:GetText();
			if ((ItemLinks[name] ~= nil) and (money ~= nil)) then
				local quantity = LootLink_HandleTooltipAddMoneyGetQuantity();
				--Print(name.." : "..quantity.." @ "..money);
				local realMoney = money;
				if ( ( LootLink_UseQuantityToDeterminePrice == 1 ) and ( quantity > 0 ) ) then
					realMoney = realMoney/quantity;
				end
				
				if(ItemLinks[name].price ~= nil ) then
					if(ItemLinks[name].price > realMoney) then
	  					ItemLinks[name].price = realMoney;
					end
				else
					ItemLinks[name].price = realMoney;
				end
			end
		end
	end
end


function LootLink_SetRefreshOnShow(toggle)
	if ( toggle ~= LootLink_RefreshOnShow ) then
		LootLink_RefreshOnShow = toggle;
	end
end

function SetLLTooltipMoney(frame, money)
	frame:AddLine(" ", 1.0, 1.0, 1.0);
	local numLines = frame:NumLines();
	local moneyFrame = getglobal(frame:GetName().."MoneyFrame");
	moneyFrame:SetPoint("LEFT", frame:GetName().."TextLeft"..numLines, "LEFT", 0, 20);
	moneyFrame:Show();
	MoneyFrame_Update(moneyFrame:GetName(), money);
	frame:SetMoneyWidth(moneyFrame:GetWidth());
end

function LL_Tooltip_ClearMoney(frame)
	local moneyFrame = getglobal(frame:GetName().."MoneyFrame");
	moneyFrame:SetPoint("LEFT", frame:GetName().."TextLeft1", "LEFT", 0, 0);
	moneyFrame:Hide();
end

function LootLinkExtension_HookLootLinkFunctions()
	LootLinkExtension_Saved_LootLinkItemButton_OnEnter = LootLinkItemButton_OnEnter;
	LootLinkItemButton_OnEnter = LootLinkExtension_LootLinkItemButton_OnEnter;
	LootLinkExtension_Saved_LootLinkItemButton_OnLeave = LootLinkItemButton_OnLeave;
	LootLinkItemButton_OnLeave = LootLinkExtension_LootLinkItemButton_OnLeave;
	LootLinkExtension_Saved_LootLink_OnShow = LootLink_OnShow;
	LootLink_OnShow = LootLinkExtension_LootLink_OnShow;
	LootLinkExtension_Saved_LootLink_ShowTooltip = LootLink_ShowTooltip;
	LootLink_ShowTooltip = LootLinkExtension_LootLink_ShowTooltip;
end

function LootLinkExtension_LootLinkItemButton_OnEnter()
	LootLinkExtension_Saved_LootLinkItemButton_OnEnter();
	local name = this:GetText();
	if (ItemLinks[name].price ~= nil) then
		--SetTooltipMoney(LootLinkTooltip, ItemLinks[name].price);
		SetLLTooltipMoney(LootLinkTooltip, ItemLinks[name].price);
	end
end

function LootLinkExtension_LootLinkItemButton_OnLeave()
	if( LootLinkFrame.TooltipButton ~= nil ) then
		LL_Tooltip_ClearMoney(LootLinkTooltip);
	end
	LootLinkExtension_Saved_LootLinkItemButton_OnLeave();
end

function LootLinkExtension_LootLink_OnShow()
	if ( LootLink_RefreshOnShow == 1 ) then
		LootLink_Refresh();
	end
	LootLinkExtension_Saved_LootLink_OnShow()
end

function LootLinkExtension_LootLink_ShowTooltip(link)
	LootLinkExtension_Saved_LootLink_ShowTooltip(link);
	local name = "LootLinkTooltipTextLeft1";
	local value = getglobal(name);
	if ( value ) then
		name = value:GetText();
		if ( ( name ) and ( strlen(name) > 0 ) ) then
			local element = LootLink_AuctionArray[name];
			LootLink_AddAuctionInfoToTooltip(element);
		end
	end
end

